# Generated by roxygen2: do not edit by hand

export("%>%")
export(":=")
export(.data)
export(build_matsindf_apply_data_frame)
export(collapse_to_matrices)
export(df_to_msg)
export(everything_except)
export(expand_to_tidy)
export(group_by_everything_except)
export(index_column)
export(mat_to_rowcolval)
export(matrix_cols)
export(matsindf_apply)
export(matsindf_apply_types)
export(rowcolval_to_mat)
export(verify_cols_missing)
importFrom(magrittr,"%>%")
importFrom(rlang,":=")
importFrom(rlang,.data)
